use egui::IconData;
use egui_extras::{Column, TableBuilder};
use libpt::log::trace;

pub mod entry;
pub mod showbox;

use self::showbox::ShowBox;

use super::*;

const ICON_RAW_PATH: &str = "file://../../../assets/img/icon-512.jpg";
const ICON_RAW: &[u8; 36525] = include_bytes!("../../../assets/img/icon-512.jpg");

impl Player {
    fn top_panel(&mut self, ui: &mut egui::Ui, ctx: &egui::Context) {
        egui::menu::bar(ui, |ui| {
            // NOTE: no File->Quit on web pages!
            let is_web = cfg!(target_arch = "wasm32");
            if !is_web {
                ui.menu_button("File", |ui| {
                    if ui.button("Info").clicked() {
                        self.show_info_window = true;
                    }
                    ui.separator();
                    if ui.button("Quit").clicked() {
                        ctx.send_viewport_cmd(egui::ViewportCommand::Close);
                    }
                });
                ui.add_space(16.0);
            }
            if self.show_info_window {
                self.info_diag(ctx);
            }

            egui::widgets::global_dark_light_mode_buttons(ui);
        });
    }

    fn info_diag(&mut self, ctx: &egui::Context) {
        trace!("rendering info dialogue");
        ctx.show_viewport_immediate(
            egui::ViewportId::from_hash_of(format!("{TITLE}: Information")),
            egui::ViewportBuilder::default()
                .with_title(format!("{TITLE}: Information"))
                .with_inner_size([500.0, 200.0]),
            |ctx, class| {
                assert!(
                    class == egui::ViewportClass::Immediate,
                    "This egui backend doesn't support multiple viewports"
                );

                egui::CentralPanel::default().show(ctx, |ui| {
                    ui.label(format!("{TITLE} v{}", env!("CARGO_PKG_VERSION")));
                    ui.hyperlink_to("Source Code\n", env!("CARGO_PKG_REPOSITORY"));
                    ui.label(format!("Author: {}", env!("CARGO_PKG_AUTHORS")));
                    ui.label(format!("License: {}", env!("CARGO_PKG_LICENSE")));
                    ui.label(format!(
                        "\n{TITLE} is free software. If you paid for this you were scammed.\n"
                    ));

                    ////////////////////////////////////////////////////////////////

                    ui.horizontal(|ui| {
                        ui.spacing_mut().item_spacing.x = 0.0;
                        ui.label("Powered by ");
                        ui.hyperlink_to("egui", "https://github.com/emilk/egui");
                        ui.label(" and ");
                        ui.hyperlink_to(
                            "eframe",
                            "https://github.com/emilk/egui/tree/master/crates/eframe",
                        );
                        ui.label(".");
                    });

                    ////////////////////////////////////////////////////////////////
                });
                if ctx.input(|i| i.viewport().close_requested()) {
                    // Tell parent viewport that we should not show next frame:
                    self.show_info_window = false;
                }
            },
        );
    }

    fn bottom_label(ui: &mut egui::Ui) {
        ui.with_layout(egui::Layout::bottom_up(egui::Align::LEFT), |ui| {
            ui.horizontal(|ui| {
                ui.spacing_mut().item_spacing.x = 0.0;
                ui.label(format!("{TITLE} v{}", env!("CARGO_PKG_VERSION")));
                ui.label(" | Powered by ");
                ui.hyperlink_to("egui", "https://github.com/emilk/egui");
                ui.label(" and ");
                ui.hyperlink_to(
                    "eframe",
                    "https://github.com/emilk/egui/tree/master/crates/eframe",
                );
                ui.label(".");
            });
            egui::warn_if_debug_build(ui);
        });
    }

    fn main_panel(&mut self, ui: &mut egui::Ui, ctx: &egui::Context) {
        let mut tb = TableBuilder::new(ui);

        let entries_per_line: usize = ctx.screen_rect().width() as usize / 256;

        for _ in 0..entries_per_line {
            tb = tb.column(Column::remainder());
        }

        tb.body(|mut body| {
            for line in self.entries().chunks(entries_per_line) {
                body.row(240.0, |mut row| {
                    for e in line {
                        row.col(|ui| {
                            ui.add(ShowBox::new(e));
                        });
                    }
                });
            }
        });
    }

    fn meta_panel(&mut self, ui: &mut egui::Ui, ctx: &egui::Context) {
        egui::menu::bar(ui, |ui| {
            // NOTE: no File->Quit on web pages!
            ui.menu_button("File", |ui| {
                if ui.button("Info").clicked() {
                    self.show_info_window = true;
                }
                ui.separator();
                if ui.button("Quit").clicked() {
                    ctx.send_viewport_cmd(egui::ViewportCommand::Close);
                }
            });
            ui.add_space(16.0);
        });
    }

    pub(crate) fn load_icon() -> IconData {
        let (icon_rgba, icon_width, icon_height) = {
            let image = image::load_from_memory(ICON_RAW)
                .expect("Failed to open icon path")
                .into_rgba8();
            let (width, height) = image.dimensions();
            let rgba = image.into_raw();
            (rgba, width, height)
        };

        IconData {
            rgba: icon_rgba,
            width: icon_width,
            height: icon_height,
        }
    }
}

impl eframe::App for Player {
    /// Called each time the UI needs repainting, which may be many times per second.
    fn update(&mut self, ctx: &egui::Context, _frame: &mut eframe::Frame) {
        egui_extras::install_image_loaders(ctx); // apperently it's okay to have this every update
                                                 // as it's only done once? sus
        egui::TopBottomPanel::top("top_panel").show(ctx, |ui| {
            self.top_panel(ui, ctx);
        });
        egui::TopBottomPanel::top("top_panel2").show(ctx, |ui| {
            self.meta_panel(ui, ctx);
        });
        egui::CentralPanel::default().show(ctx, |ui| {
            self.main_panel(ui, ctx);
        });
        egui::TopBottomPanel::bottom("bot_panel").show(ctx, Self::bottom_label);
    }
}
